//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name             : spi_main.c
//* Object                : SPi main application written in C
//* 1.0 24/01/06 SYO	  : Creation
//*----------------------------------------------------------------------------

// Include Standard files
#include "Board.h"

// #define  GLOBAL
#include "global.h"
#include "common.h"
#include "handler.h"
#include "spi_main.h"
#include "spi_master.h"
#include "spi_slave.h"

//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_SWReset
//* Object              	: SPI Software reset and check reg after reset
//* Input Parameters    : none.
//* Output Parameters  : none.
//*--------------------------------------------------------------------------------------
void AT91F_SPI_SWReset (const AT91PS_SPI pSpi)
{
	pSpi -> SPI_CR  = AT91C_SPI_SWRST; // SW reset
}



//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_En_Dis
//* Object              	: SPI Enable  Disable
//* Input Parameters    : Enable.
//* Output Parameters  : error.
//*--------------------------------------------------------------------------------------
void AT91F_SPI_En_Dis (const AT91PS_SPI pSpi, unsigned char Enable)
{
	volatile unsigned int error;

	error = 0;

	switch (Enable)
	{ 
		case ENABLE_SPI:
		pSpi -> SPI_CR  = (AT91C_SPI_SPIEN|AT91C_SPI_LASTXFER); // SPI is enable
		Wait (0xFF);
		break;

		case DISABLE_SPI:
		pSpi -> SPI_CR  = AT91C_SPI_SPIDIS; // SPI is Disable
		Wait (0xFF);
		break;

		case ENABLE_DISABLE_SPI:
		pSpi -> SPI_CR  = (AT91C_SPI_SPIDIS | AT91C_SPI_SPIEN); // Enable and disable are set @ the same time => SPI shoulb be disable
		Wait (0xFF);
		break;
	}
}


//*--------------------------------------------------------------------------------------
//* Function Name       : AT91F_Set_SPI_CSR
//* Object              : Allows programming SPI_CSR registers
//* Input Parameters    : Csr_Value.
//* Input Parameters    : Csr_Number.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
void AT91F_Set_SPI_CSR (const AT91PS_SPI pSpi, unsigned long Csr_Value, unsigned long Csr_Number)
{
	switch (Csr_Number)
	{
		case CSR0:
		pSpi -> SPI_CSR[CSR0]  = Csr_Value; // CSR0 <- Csr_Value
		break;

		case CSR1:
		pSpi -> SPI_CSR[CSR1]  = Csr_Value; // CSR1 <- Csr_Value
		break;
		
		case CSR2:
		pSpi -> SPI_CSR[CSR2]  = Csr_Value; // CSR2 <- Csr_Value
		break;

		case CSR3:
		pSpi -> SPI_CSR[CSR3]  = Csr_Value; // CSR3 <- Csr_Value
		break;

		default:
		break; 
	}
}


//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_Read_Data
//* Object              	: Read SPI_RDR
//* Input Parameters    : none.
//* Output Parameters  : Data_Recieved.
//*--------------------------------------------------------------------------------------
unsigned long AT91F_SPI_Read_Data (const AT91PS_SPI pSpi)
{
	unsigned long Data_Recieved;

	spi_Rdrf_Flag = 0;
	pSpi -> SPI_IER = AT91C_SPI_RDRF; // Disabled in the interrupt handler

	while (spi_Rdrf_Flag == 0);

	Data_Recieved = (pSpi -> SPI_RDR & 0x000FFFFF); // Read RDR and mask non used bits

	pSpi -> SPI_IER = AT91C_SPI_RDRF; // RDF re enabled (see handler)

	return Data_Recieved; // Return RDR (Read Data + PCS)
}


//*--------------------------------------------------------------------------------------
//* Function Name       : AT91F_SPI_Send_Data
//* Object              : Send a data
//* Input Parameters    : Data_To_Send.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
void AT91F_SPI_Send_Data (const AT91PS_SPI pSpi, unsigned long Data_To_Send)
{
	
        // IMPORTANT: Data_To_Send = TD (in fixed mode)
	// IMPORTANT: Data_To_Send = TD + PCS + LASTXFER (in variable mode)
	spi_Tdre_Flag = 0;

	//Data_To_Send &= 0x010FFFFF; // mask non used bits
	pSpi -> SPI_TDR = Data_To_Send;
       
  
}


//*--------------------------------------------------------------------------------------
//* Function Name       	: AT91F_SPI_Open
//* Object              	: Open the SPI0
//* Input Parameters    : none.
//* Output Parameters  : error.
//*--------------------------------------------------------------------------------------
void  AT91F_SPI_Open (void)
{
// MISO, MOSI, CLK, NPCS[3:0] are configured
// SPI (IP) clock is activated
// Interrupt Handler is configured
// SPI is resetted

	AT91PS_SPI     pSPI = AT91C_BASE_SPI;

	//  Cfp SPI PIO, PMC, ...
		// Configure SPI PIOs
  		AT91F_SPI_CfgPIO ();

	  	// Configure PMC by enabling SPI clock
  		AT91F_SPI_CfgPMC ();

		// Configure SPI interrupt
		AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_SPI, AT91C_AIC_PRIOR_HIGHEST,AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE, AT91F_SPI_Interrupt_handler);
	  	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_SPI);	

		// Reset the SPI (note it is possible to reset the SPI even if SPI is disabled)
		AT91F_SPI_SWReset (AT91C_BASE_SPI);

		spi_Txempty_Flag = 0;

		pSPI -> SPI_IER =  	(AT91C_SPI_RDRF + AT91C_SPI_TDRE + AT91C_SPI_MODF + AT91C_SPI_OVRES+ 
							 AT91C_SPI_NSSR + AT91C_SPI_TXEMPTY + AT91C_SPI_SPIENS);

		 AT91F_SPI_En_Dis (AT91C_BASE_SPI, ENABLE_SPI);

		spi_Txempty_Flag = 0;
}


